#!/usr/bin/env python
# this code is an extension of the code of 
# http://nbviewer.ipython.org/gist/harrism/f5707335f40af9463c43 
# Peter Beerli
import numpy as np
from timeit import default_timer as timer
import math
import cmath
import matplotlib.pyplot as plt

def create_jetcolorlist(max_iter,reverse=False):
  #jetcmap = plt.cm.get_cmap("cool", max_iter) #generate a map violet/purple/blue
  #jetcmap = plt.cm.get_cmap("CMRmap", max_iter) #generate a map yellow/red to darkblue 
  jetcmap = plt.cm.get_cmap("YlGnBu", max_iter) #generate a yellow to blue map 
  jet_vals = jetcmap(np.arange(max_iter)) #extract those values as an array 
  if reverse:
    return jet_vals[::-1]
  else:
    return jet_vals

def pick_jetcolor(jet_vals,max_iter, i):
  x = jet_vals[i % max_iter]
  #print x
  return x

def mapcolor(i, r, c):
  di = float(i)
  zn = cmath.sqrt(r + c);
  hue = di - cmath.log(cmath.log(np.abs(zn.real))) / cmath.log(2.0)  # 2 is escape radius
  hue = hue.real
  hue = 0.95 + 20.0 * hue
  xx = pick_jetcolor(jet_vals,max_iter,int(hue))
  return xx

def mandel(x, y, max_iters):
  c = complex(x, y)
  z = 0.0j
  for i in range(max_iters):
    z = z*z + c
    re = z.real*z.real 
    im = z.imag*z.imag
    if (re+im) >= 1024.0:
      zz = [ij for ij in mapcolor(i, re, im)]
      return zz
  return (0,0,0,1)

def create_fractal(min_x, max_x, min_y, max_y, image, iters):
  height = image.shape[0]
  width = image.shape[1]

  pixel_size_x = (max_x - min_x) / width
  pixel_size_y = (max_y - min_y) / height
    
  for x in range(width):
    real = min_x + x * pixel_size_x
    for y in range(height):
      imag = min_y + y * pixel_size_y
      color = mandel(real, imag, iters)
      image[y, x] =  color

# GLOBAL variable
jet_vals = []

if __name__ == '__main__':
  w = 1000
  h = 1500
  max_iter = 1000

  jet_vals = create_jetcolorlist(max_iter,True)
  image = np.empty((w,h,4),np.float32)
  start = timer()
  xl,xu,yl,yu = [-2.,1.,-1.0,1.0]
  #xl,xu,yl,yu = [-1.5,-0.5,-0.25,0.25]
  #xl,xu,yl,yu = [-1.3,-1.2,-0.15,0.0]
  create_fractal(xl,xu,yl,yu, image, max_iter) 
  dt = timer() - start
  print "Mandelbrot created in %f s" % dt
  ii=plt.imshow(image,interpolation='none', extent=[xl,xu,yl,yu],aspect='auto') 
  plt.show()

